 function [MA_]=MA_simul(M_,MA_,options_,oo_)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% MA_simul.m
%
% This file calculates simulation(s) with a sequence of random shocks and 
% compares the simulation with the one produced by dynare in a plot
% 
%Included in the package nonlinear_MA
%
%THIS VERSION: 1.0.1 Dec. 19, 2011
%
%Copyright: Hong Lan and Alexander Meyer-Gohde
%
%You are free to use/modify/redistribute this program so long as original
%authorship credit is given and you in no way impinge on its free
%distribution
%This software is provided as is with no guarantees of any kind
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Endogenous pruning algorithm begins here
S=oo_.exo_simul';
MA_.simul_first(:,1)=MA_.BETA_1*S(:,1);
if options_.order>=2
MA_.simul_second(:,1)=MA_.BETA_1*S(:,1)+(1/2)*MA_.BETA_2*kron([zeros(M_.endo_nbr,1);S(:,1)],[zeros(M_.endo_nbr,1);S(:,1)]);
if options_.order==3
 MA_.simul_first_sigma_2(:,1)=(1/2)*MA_.BETA_sigma_sigma_1*[zeros(M_.endo_nbr,1);S(:,1)];
MA_.simul_third(:,1)=MA_.BETA_1*S(:,1)+(1/2)*MA_.BETA_2*kron([zeros(M_.endo_nbr,1);S(:,1)],[zeros(M_.endo_nbr,1);S(:,1)])...
    +(1/6)*MA_.BETA_3_1*kron([zeros(M_.endo_nbr,1);S(:,1)],kron([zeros(M_.endo_nbr,1);S(:,1)],[zeros(M_.endo_nbr,1);S(:,1)]))...
    +MA_.BETA_3_3*kron([zeros(M_.endo_nbr,1);S(:,1)],[zeros(M_.endo_nbr,1);(1/2)*kron([zeros(M_.endo_nbr,1);S(:,1)],[zeros(M_.endo_nbr,1);S(:,1)])])...
    +(1/2)*MA_.BETA_sigma_sigma_1*[zeros(M_.endo_nbr,1);S(:,1)];

end
end
for t=2:options_.periods
    MA_.simul_first(:,t)=MA_.ALPHA*MA_.simul_first(:,t-1)+MA_.BETA_1*S(:,t);
if options_.order>=2
MA_.simul_second(:,t)=MA_.ALPHA*MA_.simul_second(:,t-1)+MA_.BETA_1*S(:,t)+(1/2)*MA_.BETA_2*kron([MA_.simul_first(:,t-1);S(:,t)],[MA_.simul_first(:,t-1);S(:,t)]);
if options_.order==3
    MA_.simul_first_sigma_2(:,t)=MA_.ALPHA*MA_.simul_first_sigma_2(:,t-1)+(1/2)*MA_.BETA_sigma_sigma_1*[MA_.simul_first(:,t-1);S(:,t)];
MA_.simul_third(:,t)=MA_.ALPHA*MA_.simul_third(:,t-1)+MA_.BETA_1*S(:,t)+(1/2)*MA_.BETA_2*kron([MA_.simul_first(:,t-1);S(:,t)],[MA_.simul_first(:,t-1);S(:,t)])...
    +(1/6)*MA_.BETA_3_1*kron([MA_.simul_first(:,t-1);S(:,t)],kron([MA_.simul_first(:,t-1);S(:,t)],[MA_.simul_first(:,t-1);S(:,t)]))...
    +MA_.BETA_3_3*kron([MA_.simul_first(:,t-1);S(:,t)],[MA_.simul_second(:,t-1)-MA_.simul_first(:,t-1);(1/2)*kron([MA_.simul_first(:,t-1);S(:,t)],[MA_.simul_first(:,t-1);S(:,t)])])...
    +(1/2)*MA_.BETA_sigma_sigma_1*[MA_.simul_first(:,t-1);S(:,t)];

end
end
end
%Endogenous pruning algorithm ends here

%Begin combining and plotting
for ii=1:M_.endo_nbr
    if options_.order==1
MA_.simul_total(ii,:)=MA_.simul_first(ii,:)+oo_.dr.ys(ii);
if MA_.plot_simulations==1
  figure('Units','characters','Position',[1 1 240 70]);
    plot(1+options_.drop:options_.periods,MA_.simul_total(ii,1+options_.drop:end),'b')
    eval(sprintf('title(''Simulation of %s'')',M_.endo_names(ii,:)))
     ylabel('Value');
    xlabel('Periods');
    legend({'First-Order Accurate'},'Location','Best')
end
    elseif options_.order==2
        MA_.simul_total(ii,:)=oo_.dr.ys(ii)+MA_.Y_sigma_2(ii)+MA_.simul_second(ii,:);
        if MA_.plot_simulations==1
        figure('Units','characters','Position',[1 1 240 70]);
    subplot(3,3,[1:6]);plot(1+options_.drop:options_.periods,MA_.simul_total(ii,1+options_.drop:end),'b',1+options_.drop:options_.periods,MA_.simul_first(ii,1+options_.drop:end)+oo_.dr.ys(ii),'b-.',1+options_.drop:options_.periods,oo_.endo_simul(ii,1+options_.drop:options_.periods),'k--')
    eval(sprintf('title(''Simulation of %s'')',M_.endo_names(ii,:)))
     ylabel('Value');
    xlabel('Periods');
    if options_.pruning==1
    legend({'Second-Order Accurate','First-Order Accurate','Dynare: Pruned Second Order'},'Location','Best')
    else
    legend({'Second-Order Accurate','First-Order Accurate','Dynare: Second Order'},'Location','Best')
    end
       subplot(3,3,[7]); plot(1+options_.drop:options_.periods,MA_.simul_first(ii,1+options_.drop:end),'b')
    title('First-Order Component')
    ylabel('Deviations');
    xlabel('Periods');
    hold on
     plot(1+options_.drop:options_.periods,repmat(0, [options_.periods-options_.drop]),'k-')
     hold off
      subplot(3,3,[8]); plot(1+options_.drop:options_.periods,MA_.simul_second(ii,1+options_.drop:end)-MA_.simul_first(ii,1+options_.drop:end),'b')
    title('Second-Order Component')
    ylabel('Deviations');
    xlabel('Periods');
    hold on
     plot(1+options_.drop:options_.periods,repmat(0, options_.periods-options_.drop),'k-')
     hold off
      subplot(3,3,[9]); plot(1+options_.drop:options_.periods,repmat(oo_.dr.ys(ii), [1 options_.periods-options_.drop]),'b',1+options_.drop:options_.periods,repmat(oo_.dr.ys(ii)+MA_.Y_sigma_2(ii), [1 options_.periods-options_.drop]),'b-.')
    title('Steady-State and Constant')
    legend({'Steady-State','plus Risk Adjutsment'},'Location','Best')  
        end
    elseif options_.order==3
                MA_.simul_total(ii,:)=oo_.dr.ys(ii)+MA_.Y_sigma_2(ii)+MA_.simul_third(ii,:);
                if MA_.plot_simulations==1
        figure('Units','characters','Position',[1 1 240 70]);
     subplot(4,3,[1:6]);plot(1+options_.drop:options_.periods,MA_.simul_total(ii,1+options_.drop:end),'b',1+options_.drop:options_.periods,oo_.dr.ys(ii)+MA_.Y_sigma_2(ii)+MA_.simul_second(ii,1+options_.drop:end),'b--',1+options_.drop:options_.periods,MA_.simul_first(ii,1+options_.drop:end)+oo_.dr.ys(ii),'b-.',1+options_.drop:options_.periods,oo_.endo_simul(ii,1+options_.drop:options_.periods),'k--')
    eval(sprintf('title(''Simulation of %s'')',M_.endo_names(ii,:)))
     ylabel('Value');
    xlabel('Periods');
    legend({'Third-Order Accurate','Second-Order Accurate','First-Order Accurate','Dynare: Third Order'},'Location','Best')
       subplot(4,3,[7]); plot(1+options_.drop:options_.periods,MA_.simul_first(ii,1+options_.drop:end),'b')
    title('First-Order Component')
    ylabel('Deviations');
    xlabel('Periods');
    hold on
     plot([1+options_.drop:options_.periods],repmat(0, [options_.periods-options_.drop]),'k-')
     hold off
      subplot(4,3,[8]); plot(1+options_.drop:options_.periods,MA_.simul_second(ii,1+options_.drop:end)-MA_.simul_first(ii,1+options_.drop:end),'b')
    title('Second-Order Component')
    ylabel('Deviations');
    xlabel('Periods');
    hold on
     plot([1+options_.drop:options_.periods],repmat(0, options_.periods-options_.drop),'k-')
     hold off
     subplot(4,3,[10]); plot(1+options_.drop:options_.periods,MA_.simul_first_sigma_2(ii,1+options_.drop:end),'b')
    title('Risk Correction to First-Order')
    ylabel('Deviations');
    xlabel('Periods');
    hold on
     plot([1+options_.drop:options_.periods],repmat(0, [options_.periods-options_.drop]),'k-')
     hold off
      subplot(4,3,[11]); plot(1+options_.drop:options_.periods,MA_.simul_third(ii,1+options_.drop:end)-MA_.simul_first_sigma_2(ii,1+options_.drop:end)-MA_.simul_second(ii,1+options_.drop:end),'b')
    title('Third-Order Component')
    ylabel('Deviations');
    xlabel('Periods');
    hold on
     plot([1+options_.drop:options_.periods],repmat(0, [options_.periods-options_.drop]),'k-')
     hold off
      subplot(4,3,[9]); plot(1+options_.drop:options_.periods,repmat(oo_.dr.ys(ii), [1 options_.periods-options_.drop]),'b',1+options_.drop:options_.periods,repmat(oo_.dr.ys(ii)+MA_.Y_sigma_2(ii), [1 options_.periods-options_.drop]),'b-.')
    title('Steady-State and Constant')
    legend({'Steady-State','plus Risk Adjutsment'},'Location','Best')   
                end
    end
end
