   function [MA_]=third_order(M_,MA_)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% third_order.m
%
% This file produces the coefficients for a third-order approximation in 
% addition to those produced by first_order.m and second_order.m
%
%Included in the package nonlinear_MA
%
%THIS VERSION: 1.0.1 Dec. 19, 2011
%
%Copyright: Hong Lan and Alexander Meyer-Gohde
%
%You are free to use/modify/redistribute this program so long as original
%authorship credit is given and you in no way impinge on its free
%distribution
%This software is provided as is with no guarantees of any kind
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



% build up shifting matrix gamma(2), where (2) indicates that this shifting
% matrix bridges second order solution to the third order. 
% build up gamma(2), such that
%[y_(j-1)(i-1)',y_(j)(i)',y_(j+1)(i+1)',0]'=gamma(2)*[y_(j-1)(i-1)' kron([y_(j-1)', U_(j)']',[y_(i-1)', U_(i)']')']'. 
MA_.GAMMA_2 = [ eye(M_.endo_nbr)               zeros(M_.endo_nbr,(M_.exo_nbr+M_.endo_nbr)^2)
                                MA_.ALPHA                         MA_.BETA_2
                                MA_.ALPHA^2                     MA_.BETA_2*kron(MA_.DELTA_1,MA_.DELTA_1)+MA_.ALPHA*MA_.BETA_2
                       zeros(M_.exo_nbr,M_.endo_nbr)  zeros(M_.exo_nbr,(M_.exo_nbr+M_.endo_nbr)^2)        ];

% build up shifting matrix delta(2), such that
% [y_(j)(i)' kron([y_(j)', U_(j+1)']',[y_(i)', U_(i+1)']')']'=delta(2)*[y_(j-1)(i-1)' kron([y_(j-1)', U_(j)']',[y_(i-1)', U_(i)']')']'. 
MA_.DELTA_2 = [ MA_.ALPHA                       MA_.BETA_2
                       zeros((M_.exo_nbr+M_.endo_nbr)^2,M_.endo_nbr)   kron(MA_.DELTA_1,MA_.DELTA_1)  ];

% split MA_.BETA_3 into 3 problems
[waste, MA_.BETA_3_1] =gensylv(3,MA_.syl_AA,MA_.syl_BB,MA_.DELTA_1,-MA_.df3*kron(MA_.g_picker*MA_.GAMMA_1,kron(MA_.g_picker*MA_.GAMMA_1,MA_.g_picker*MA_.GAMMA_1)));
BETA_3_2=sylvester_doubling(-MA_.syl_AA\MA_.syl_BB,MA_.DELTA_2,MA_.DELTA_1,MA_.syl_AA\MA_.df2*kron(MA_.g_picker*MA_.GAMMA_2,MA_.g_picker*MA_.GAMMA_1),MA_.condn);
MA_.BETA_3_3=reshape(BETA_3_2',[  (M_.exo_nbr+M_.endo_nbr) (M_.endo_nbr+(M_.exo_nbr+M_.endo_nbr)^2) M_.endo_nbr]);
MA_.BETA_3_3=permute(MA_.BETA_3_3,[ 3 2 1]);
MA_.BETA_3_3=reshape(MA_.BETA_3_3, [M_.endo_nbr (M_.endo_nbr+(M_.exo_nbr+M_.endo_nbr)^2)*(M_.exo_nbr+M_.endo_nbr)]);
% puting the pieces together
MA_.BETA_3=[MA_.BETA_3_1,BETA_3_2,MA_.BETA_3_3,MA_.BETA_3_3];

%Zero out numbers numerically identical to zero according to the conditioning criterion
MA_.BETA_3(abs(MA_.BETA_3)<MA_.condn)=0;
MA_.BETA_3_3(abs(MA_.BETA_3_3)<MA_.condn)=0;
MA_.BETA_3_1(abs(MA_.BETA_3_1)<MA_.condn)=0;

%Now proceed to the third-order correction of the first-order kernel
MA_.GAMMA_tilde=[zeros(2*M_.endo_nbr,M_.endo_nbr);eye(M_.endo_nbr,M_.endo_nbr); zeros(M_.exo_nbr,M_.endo_nbr)];
%The constant is
%syl_DD =-((MA_.df3*kron(kron(MA_.g_picker*MA_.Y_eps_1,MA_.g_picker*MA_.Y_eps_1),MA_.g_picker*MA_.GAMMA_1)+MA_.df2*(kron(MA_.g_picker*MA_.Y_eps_2,MA_.g_picker*MA_.GAMMA_1))+2*MA_.df2*kron(MA_.g_picker*MA_.Y_eps_1,MA_.g_picker*MA_.GAMMA_tilde*MA_.BETA_2*kron([zeros(M_.endo_nbr,M_.exo_nbr);eye(M_.exo_nbr)],eye(M_.exo_nbr+M_.endo_nbr)))+MA_.df1*MA_.g_picker*MA_.GAMMA_tilde*MA_.BETA_3*[eye((M_.exo_nbr+M_.endo_nbr)^3);kron([zeros(M_.endo_nbr,(M_.exo_nbr+M_.endo_nbr)^2);eye((M_.exo_nbr+M_.endo_nbr)^2)],eye(M_.exo_nbr+M_.endo_nbr));kron(eye(M_.exo_nbr+M_.endo_nbr),[zeros(M_.endo_nbr,(M_.exo_nbr+M_.endo_nbr)^2);eye((M_.exo_nbr+M_.endo_nbr)^2)])*commutation(M_.exo_nbr+M_.endo_nbr,(M_.exo_nbr+M_.endo_nbr)^2)*kron(eye(M_.exo_nbr+M_.endo_nbr),commutation(M_.exo_nbr+M_.endo_nbr,M_.exo_nbr+M_.endo_nbr));kron(eye(M_.exo_nbr+M_.endo_nbr),[zeros(M_.endo_nbr,(M_.exo_nbr+M_.endo_nbr)^2);eye((M_.exo_nbr+M_.endo_nbr)^2)])]*kron([zeros(M_.endo_nbr,M_.exo_nbr);eye(M_.exo_nbr)],kron([zeros(M_.endo_nbr,M_.exo_nbr);eye(M_.exo_nbr)],eye(M_.exo_nbr+M_.endo_nbr))))*kron(vec(M_.Sigma_e),eye(M_.exo_nbr+M_.endo_nbr))+kron(MA_.g_picker*[MA_.Y_sigma_2;MA_.Y_sigma_2;MA_.Y_sigma_2;zeros(M_.exo_nbr,1)],MA_.g_picker*MA_.GAMMA_1))*Sj
[waste, MA_.BETA_sigma_sigma_1] =gensylv(1,MA_.syl_AA,MA_.syl_BB,MA_.DELTA_1,-((MA_.df3*kron(kron(MA_.g_picker*MA_.Y_eps_1,MA_.g_picker*MA_.Y_eps_1),MA_.g_picker*MA_.GAMMA_1)+MA_.df2*(kron(MA_.g_picker*MA_.Y_eps_2,MA_.g_picker*MA_.GAMMA_1))+2*MA_.df2*kron(MA_.g_picker*MA_.Y_eps_1,MA_.g_picker*MA_.GAMMA_tilde*MA_.BETA_2*kron([zeros(M_.endo_nbr,M_.exo_nbr);eye(M_.exo_nbr)],MA_.DELTA_1))+MA_.df1*MA_.g_picker*MA_.GAMMA_tilde*MA_.BETA_3*[eye((M_.exo_nbr+M_.endo_nbr)^3);kron([zeros(M_.endo_nbr,(M_.exo_nbr+M_.endo_nbr)^2);eye((M_.exo_nbr+M_.endo_nbr)^2)],eye(M_.exo_nbr+M_.endo_nbr));kron(eye(M_.exo_nbr+M_.endo_nbr),[zeros(M_.endo_nbr,(M_.exo_nbr+M_.endo_nbr)^2);eye((M_.exo_nbr+M_.endo_nbr)^2)])*commutation(M_.exo_nbr+M_.endo_nbr,(M_.exo_nbr+M_.endo_nbr)^2)*kron(eye(M_.exo_nbr+M_.endo_nbr),commutation(M_.exo_nbr+M_.endo_nbr,M_.exo_nbr+M_.endo_nbr));kron(eye(M_.exo_nbr+M_.endo_nbr),[zeros(M_.endo_nbr,(M_.exo_nbr+M_.endo_nbr)^2);eye((M_.exo_nbr+M_.endo_nbr)^2)])]*kron([zeros(M_.endo_nbr,M_.exo_nbr);eye(M_.exo_nbr)],kron([zeros(M_.endo_nbr,M_.exo_nbr);eye(M_.exo_nbr)],MA_.DELTA_1)))*kron(vec(M_.Sigma_e),eye(M_.exo_nbr+M_.endo_nbr))+MA_.df2*kron(MA_.g_picker*[MA_.Y_sigma_2;MA_.Y_sigma_2;MA_.Y_sigma_2;zeros(M_.exo_nbr,1)],MA_.g_picker*MA_.GAMMA_1)));
%Zero out numbers numerically identical to zero according to the conditioning criterion
MA_.BETA_sigma_sigma_1(abs(MA_.BETA_sigma_sigma_1)<MA_.condn)=0;